﻿#Region " Class Imports "

Imports System
Imports System.Xml
Imports System.IO

#End Region

Public Class DSOXMLReader

#Region " Class Scoped Variables "

    Private moCapturedData As New SampleLoader.CapturedData

#End Region

#Region " Private Methods "

    Private Sub DisplayProfileData()

        Try
            txtTriggerMode.Text = moCapturedData.triggerMode
            txtTriggerKind.Text = moCapturedData.triggerKind
            txtTriggerLevel.Text = moCapturedData.triggerLevel
            txtTriggerSensitivity.Text = moCapturedData.triggerSensitivity
            txtTriggerIndex.Text = moCapturedData.triggerIndex.ToString
            txtSampleCount.Text = moCapturedData.sampleCount.ToString
            txtTimeRange.Text = moCapturedData.timeRange.ToString

            StartPoint.Maximum = moCapturedData.sampleCount - 2
            StartPoint.Value = 0

            txtMaxVal.Text = moCapturedData.MaxValue.ToString
            txtMinVal.Text = moCapturedData.MinValue.ToString

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try

    End Sub

    Private Sub DrawTrace()
        Try

            If moCapturedData.sampleCount > 1 Then


                Dim SampleRate As Double = moCapturedData.timeRange / moCapturedData.sampleCount

                Dim xMultiplier As Double = 10 ^ 6
                Dim yMultiplier As Double = 1000


                Dim LastX As Double
                Dim lasty As Integer
                Dim X As Double
                Dim Y As Integer
                Dim TraceCount As Integer = 0



                Select Case yScale.SelectedIndex

                    Case Is = 0
                        yMultiplier = yMultiplier * 100
                    Case Is = 1
                        yMultiplier = yMultiplier * 10
                    Case Is = 2
                        yMultiplier = yMultiplier
                    Case Is = 3
                        yMultiplier = yMultiplier / 10
                    Case Is = 4
                        yMultiplier = yMultiplier / 100
                    Case Is = 5
                        yMultiplier = yMultiplier / 1000

                End Select

                Select Case xScale.SelectedIndex

                    Case Is = 0
                        xMultiplier = xMultiplier * 100
                    Case Is = 1
                        xMultiplier = xMultiplier * 10
                    Case Is = 2
                        xMultiplier = xMultiplier * 1
                    Case Is = 3
                        xMultiplier = xMultiplier / 100
                    Case Is = 4
                        xMultiplier = xMultiplier / 1000
                    Case Is = 5
                        xMultiplier = xMultiplier / 10000

                End Select


                xMultiplier = xMultiplier / xScaleKnob.value

                yMultiplier = yMultiplier / yScaleKnob.value

                Using oScreen As System.Drawing.Graphics = Screen.CreateGraphics()

                    For Each oTrace As SampleLoader.Points In moCapturedData.Traces

                        Dim oColour As System.Drawing.Color

                        Select Case TraceCount

                            Case Is = 0
                                oColour = ScopeTraceColour1.BackColor
                            Case Is = 1
                                oColour = ScopeTraceColour2.BackColor
                            Case Is = 2
                                oColour = ScopeTraceColour3.BackColor
                            Case Is = 3
                                oColour = ScopeTraceColour4.BackColor
                        End Select


                        Using myPen As System.Drawing.Pen = New System.Drawing.Pen(oColour, 1)
                            lasty = 500 - (CInt(oTrace.Value(StartPoint.Value).Val * yMultiplier) + Offset.Value)
                            LastX = 0
                            X = 0
                            For ReadSamples As Integer = StartPoint.Value To moCapturedData.sampleCount - 1
                                If LastX < 1000 Then
                                    Y = 500 - (CInt(oTrace.Value(ReadSamples).Val * yMultiplier) + Offset.Value)
                                    oScreen.DrawLine(myPen, CInt(LastX), lasty, CInt(X), Y)
                                    LastX = X
                                    lasty = Y
                                    X += (xMultiplier * SampleRate)
                                End If
                            Next
                        End Using

                        TraceCount += 1

                    Next



                    Using myPen As System.Drawing.Pen = New System.Drawing.Pen(Brushes.LightGoldenrodYellow, 1)
                        Dim TriggerMV As Double = GetTriggerIn_mV(moCapturedData.triggerLevel) / 1000
                        Y = 500 - CInt((TriggerMV * yMultiplier) + Offset.Value)
                        oScreen.DrawLine(myPen, 0, Y, 1000, Y)
                    End Using

                End Using


                txtTimeIndex.Text = (CDbl(StartPoint.Value * SampleRate)).ToString

                txtOffset.Text = (-((250 - Offset.Value) / yMultiplier)).ToString



            End If
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try


    End Sub

    Private Function GetTriggerIn_mV(ByVal strTrigger As String) As Double

        GetTriggerIn_mV = 0

        Dim strNumbePart As String = String.Empty
        Dim strRangePart As String = String.Empty
        Dim strPart As String = String.Empty
        Dim intCounter As Integer = 0

        Try
            For intCounter = 0 To Len(strTrigger) - 1
                strPart = strTrigger.Substring(intCounter, 1)
                If IsNumeric(strPart) OrElse strPart = "." Then
                    strNumbePart &= strPart
                Else
                    Exit For
                End If
            Next

            strRangePart = strTrigger.Substring(intCounter)

            Dim mulitplyer As Double = 0

            Select Case strRangePart.Trim.ToUpper
                Case Is = "UV"
                    mulitplyer = 0
                Case Is = "MV"
                    mulitplyer = 100
                Case Is = "V"
                    mulitplyer = 1000
            End Select


            GetTriggerIn_mV = (CDbl(strNumbePart) * mulitplyer)

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try

    End Function

    Private Sub DrawFFT()

        Dim DataValues(moCapturedData.sampleCount) As Double

        Dim TraceCount As Integer = 0

        Try
            For Each oTrace As SampleLoader.Points In moCapturedData.Traces

                Dim oColour As System.Drawing.Color

                Select Case TraceCount

                    Case Is = 0
                        oColour = AnalysisTraceColour1.BackColor
                    Case Is = 1
                        oColour = AnalysisTraceColour2.BackColor
                    Case Is = 2
                        oColour = AnalysisTraceColour3.BackColor
                    Case Is = 3
                        oColour = AnalysisTraceColour4.BackColor
                End Select


                For intCounter As Integer = 0 To moCapturedData.sampleCount - 1
                    DataValues(intCounter) = oTrace.Value(intCounter).Val
                Next

                Dim TransformedData() As Double = FFT.Transform(DataValues).Decibel

                RenderEngine.RenderFrequencyDomain(FFTDisplayDecibel, CInt(moCapturedData.sampleCount / moCapturedData.timeRange), FFT.Transform(DataValues).Decibel, oColour)
                RenderEngine.RenderFrequencyDomain(FFTDisplayMagnitude, CInt(moCapturedData.sampleCount / moCapturedData.timeRange), FFT.Transform(DataValues).Magnitude, oColour)

                TraceCount += 1

            Next

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try



    End Sub

    Private Sub DrawGrid()


        Try


            Using oScreen As System.Drawing.Graphics = Screen.CreateGraphics()

                oScreen.Clear(Color.Black)

                Using myPen As System.Drawing.Pen = New System.Drawing.Pen(Brushes.SlateGray, 1)
                    For intX As Integer = 100 To 900 Step 100
                        oScreen.DrawLine(myPen, intX, 0, intX, 500)
                    Next
                    For intY As Integer = 50 To 450 Step 100
                        oScreen.DrawLine(myPen, 0, intY, 1000, intY)
                    Next
                End Using

                Using myPen As System.Drawing.Pen = New System.Drawing.Pen(Brushes.Red, 1)
                    For intX As Integer = 100 To 900 Step 100
                        oScreen.DrawLine(myPen, 0, 500 - Offset.Value, 1000, 500 - Offset.Value)
                    Next
                End Using

            End Using



            Dim strTimeValue As String = String.Empty
            Dim strVoltageValue As String = String.Empty



            If yScale.SelectedItem IsNot Nothing Then



                Select Case yScale.SelectedIndex

                    Case Is = 0
                        strVoltageValue = " mV Div"
                    Case Is = 1
                        strVoltageValue = "0 mV Div"
                    Case Is = 2
                        strVoltageValue = "00 mV Div"
                    Case Is = 3, 4, 5
                        strVoltageValue = " V Div"
                    Case Is = 4
                        strVoltageValue = "0 V Div"
                    Case Is = 5
                        strVoltageValue = "00 V Div"

                End Select

            End If



            If xScale.SelectedItem IsNot Nothing Then

                Select Case xScale.SelectedIndex

                    Case Is = 0
                        strTimeValue = " uS Div"
                    Case Is = 1
                        strTimeValue = "0 uS Div"
                    Case Is = 2
                        strTimeValue = "00 uS Div"
                    Case Is = 3
                        strTimeValue = " mS Div"
                    Case Is = 4
                        strTimeValue = "0 mS Div"
                    Case Is = 5
                        strTimeValue = "00 mS Div"

                End Select

            End If



            txtYScaleKnobValue.Text = yScaleKnob.value.ToString & strVoltageValue
            vScale.Text = txtYScaleKnobValue.Text

            txtXScaleKnobValue.Text = xScaleKnob.value.ToString & strTimeValue
            HScale.Text = txtXScaleKnobValue.Text


        Catch ex As Exception
            MsgBox(ex.Message)
        End Try


    End Sub

#End Region

#Region " Form Events "


    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        xScale.SelectedIndex = 0
        yScale.SelectedIndex = 0
        DrawGrid()
        ' PUT THIS IN UNTIL WE HAVE A QUAD XML FILE TO PLAY WITH TO REMOVE THE CONFIGURATION FOR THE TRACES
        TabControl1.TabPages.Remove(TabControl1.TabPages.Item(2))
    End Sub

    Private Sub xScale_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles xScale.SelectedIndexChanged
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub yScale_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles yScale.SelectedIndexChanged
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub Offset_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Offset.Scroll
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub StartPoint_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartPoint.Scroll
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub cmdScopeTraceColour1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdScopeTraceColour1.Click
        ColorDialog1.Color = ScopeTraceColour1.BackColor
        ColorDialog1.ShowDialog()
        ScopeTraceColour1.BackColor = ColorDialog1.Color
    End Sub

    Private Sub cmdScopeTraceColour2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdScopeTraceColour2.Click
        ColorDialog1.Color = ScopeTraceColour2.BackColor
        ColorDialog1.ShowDialog()
        ScopeTraceColour2.BackColor = ColorDialog1.Color
    End Sub

    Private Sub cmdScopeTraceColour3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdScopeTraceColour3.Click
        ColorDialog1.Color = ScopeTraceColour3.BackColor
        ColorDialog1.ShowDialog()
        ScopeTraceColour3.BackColor = ColorDialog1.Color
    End Sub

    Private Sub cmdScopeTraceColour4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdScopeTraceColour4.Click
        ColorDialog1.Color = ScopeTraceColour4.BackColor
        ColorDialog1.ShowDialog()
        ScopeTraceColour4.BackColor = ColorDialog1.Color
    End Sub

    Private Sub cmdAnalysisTraceColour1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAnalysisTraceColour1.Click
        ColorDialog1.Color = AnalysisTraceColour1.BackColor
        ColorDialog1.ShowDialog()
        AnalysisTraceColour1.BackColor = ColorDialog1.Color
    End Sub

    Private Sub cmdAnalysisTraceColour2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAnalysisTraceColour2.Click
        ColorDialog1.Color = AnalysisTraceColour2.BackColor
        ColorDialog1.ShowDialog()
        AnalysisTraceColour2.BackColor = ColorDialog1.Color
    End Sub

    Private Sub cmdAnalysisTraceColour3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAnalysisTraceColour3.Click
        ColorDialog1.Color = AnalysisTraceColour3.BackColor
        ColorDialog1.ShowDialog()
        AnalysisTraceColour3.BackColor = ColorDialog1.Color
    End Sub

    Private Sub cmdAnalysisTraceColour4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAnalysisTraceColour4.Click
        ColorDialog1.Color = AnalysisTraceColour3.BackColor
        ColorDialog1.ShowDialog()
        AnalysisTraceColour3.BackColor = ColorDialog1.Color
    End Sub

    Private Sub cmdFindMin_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdFindMin.Click
        StartPoint.Value = moCapturedData.MinIndex
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub cmdFindMax_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdFindMax.Click
        StartPoint.Value = moCapturedData.MaxIndex
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub Screen_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Screen.Paint
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub cmdTriggerFind_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdTriggerFind.Click
        StartPoint.Value = moCapturedData.triggerIndex
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub XScaleKnob_ValueChanged(ByVal Value As Integer) Handles xScaleKnob.ValueChanged
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub YScaleKnob_ValueChanged(ByVal Value As Integer) Handles yScaleKnob.ValueChanged
        DrawGrid()
        DrawTrace()
    End Sub

    Private Sub OpenToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OpenToolStripMenuItem.Click
        Dim oResult As DialogResult = dlgFileOpen.ShowDialog
    End Sub

    Private Sub dlgFileOpen_FileOk(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles dlgFileOpen.FileOk
        Try

            Using oLoader As New SampleLoader
                moCapturedData = oLoader.LoadXMLFile(dlgFileOpen.FileName)
            End Using

            If moCapturedData.ValidData Then
                DisplayProfileData()
                DrawGrid()
                DrawTrace()
                DrawFFT()
            End If

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try

    End Sub


#End Region

End Class
