﻿
Class SignalGenerator

    Private _waveForm As String = "Sine"
    Private _amplitude As Double = 128.0
    Private _samplingRate As Double = 44100
    Private _frequency As Double = 5000.0
    Private _dcLevel As Double = 0.0
    Private _noise As Double = 0.0
    Private _samples As Integer = 16384
    Private _addDCLevel As Boolean = False
    Private _addNoise As Boolean = False

    Public Sub SetWaveform(ByVal waveForm As String)
        _waveForm = waveForm
    End Sub

    Public Function GetWaveform() As [String]
        Return _waveForm
    End Function

    Public Sub SetAmplitude(ByVal amplitude As Double)
        _amplitude = amplitude
    End Sub

    Public Function GetAmplitude() As Double
        Return _amplitude
    End Function

    Public Sub SetFrequency(ByVal frequency As Double)
        _frequency = frequency
    End Sub

    Public Function GetFrequency() As Double
        Return _frequency
    End Function

    Public Sub SetSamplingRate(ByVal rate As Double)
        _samplingRate = rate
    End Sub

    Public Function GetSamplingRate() As Double
        Return _samplingRate
    End Function

    Public Sub SetSamples(ByVal samples As Integer)
        _samples = samples
    End Sub

    Public Function GetSamples() As Integer
        Return _samples
    End Function

    Public Sub SetDCLevel(ByVal dc As Double)
        _dcLevel = dc
    End Sub

    Public Function GetDCLevel() As Double
        Return _dcLevel
    End Function

    Public Sub SetNoise(ByVal noise As Double)
        _noise = noise
    End Sub

    Public Function GetNoise() As Double
        Return _noise
    End Function

    Public Sub SetDCLevelState(ByVal dcstate As Boolean)
        _addDCLevel = dcstate
    End Sub

    Public Function IsDCLevel() As Boolean
        Return _addDCLevel
    End Function

    Public Sub SetNoiseState(ByVal noisestate As Boolean)
        _addNoise = noisestate
    End Sub

    Public Function IsNoise() As Boolean
        Return _addNoise
    End Function

    Public Function GenerateSignal() As Double()
        Dim values As Double() = New Double(_samples - 1) {}
        If _waveForm.Equals("Sine") Then
            Dim theta As Double = 2.0 * Math.PI * _frequency / _samplingRate
            For i As Integer = 0 To _samples - 1
                values(i) = _amplitude * Math.Sin(i * theta)
            Next
        End If
        If _waveForm.Equals("Cosine") Then
            Dim theta As Double = 2.0F * CDbl(Math.PI) * _frequency / _samplingRate
            For i As Integer = 0 To _samples - 1
                values(i) = _amplitude * Math.Cos(i * theta)
            Next
        End If
        If _waveForm.Equals("Square") Then
            Dim p As Double = 2.0 * _frequency / _samplingRate
            For i As Integer = 0 To _samples - 1
                values(i) = If(Math.Round(i * p) Mod 2 = 0, _amplitude, -_amplitude)
            Next
        End If
        If _waveForm.Equals("Triangular") Then
            Dim p As Double = 2.0 * _frequency / _samplingRate
            For i As Integer = 0 To _samples - 1
                Dim ip As Integer = CInt(Math.Round(i * p))
                values(i) = 2.0 * _amplitude * (1 - 2 * (ip Mod 2)) * (i * p - ip)
            Next
        End If
        If _waveForm.Equals("Sawtooth") Then
            For i As Integer = 0 To _samples - 1
                Dim q As Double = i * _frequency / _samplingRate
                values(i) = 2.0 * _amplitude * (q - Math.Round(q))
            Next
        End If
        If _addDCLevel Then
            For i As Integer = 0 To _samples - 1
                values(i) += _dcLevel
            Next
        End If
        If _addNoise Then
            Dim r As New Random()
            For i As Integer = 0 To _samples - 1
                values(i) += _noise * r.[Next]()
            Next
        End If
        Return values
    End Function

End Class
