﻿Imports System.Drawing.Imaging

Public Class RenderEngine

    Public Shared Sub RenderFrequencyDomain(ByRef pictureBox As PictureBox, ByVal SampleFrequency As Integer, ByRef FFTResults() As Double, ByVal Pencolour As Color)
        ' Set up for drawing
        Dim canvas As New Bitmap(pictureBox.Width, pictureBox.Height)
        Dim offScreenDC As Graphics = Graphics.FromImage(canvas)
        Dim brush As SolidBrush = New System.Drawing.SolidBrush(Color.FromArgb(128, 255, 255, 255))
        Dim pen As Pen = New System.Drawing.Pen(Color.Aqua)
        Dim font As New Font("Arial", 12)

        ' Determine channnel boundries
        Dim width As Integer = canvas.Width
        Dim height As Integer = canvas.Height

        Dim min As Double = Double.MaxValue
        Dim minHz As Double = 0
        Dim max As Double = Double.MinValue
        Dim maxHz As Double = 0
        Dim range As Double = 0
        Dim scale As Double = 0
        Dim scaleHz As Double = CDbl(SampleFrequency \ 2) / CDbl(FFTResults.Length)

        ' get left min/max
        For x As Integer = 0 To FFTResults.Length - 1
            Dim amplitude As Double = FFTResults(x)
            If min > amplitude AndAlso amplitude <> Double.NegativeInfinity Then
                min = amplitude
                minHz = CDbl(x) * scaleHz
            End If
            If max < amplitude AndAlso amplitude <> Double.PositiveInfinity Then
                max = amplitude
                maxHz = CDbl(x) * scaleHz
            End If
        Next

        ' get right range
        If min < 0 OrElse max < 0 Then
            If min < 0 AndAlso max < 0 Then
                range = max - min
            Else
                range = Math.Abs(min) + max
            End If
        Else
            range = max - min
        End If
        scale = range / height

        ' draw right channel
        For xAxis As Integer = 0 To width - 1
            Dim amplitude As Double = CDbl(FFTResults(CInt(Math.Truncate((CDbl(FFTResults.Length) / CDbl(width)) * xAxis))))
            If amplitude = Double.NegativeInfinity OrElse amplitude = Double.PositiveInfinity OrElse amplitude = Double.MinValue OrElse amplitude = Double.MaxValue Then
                amplitude = 0
            End If
            Dim yAxis As Integer
            If amplitude < 0 Then
                yAxis = CInt(Math.Truncate(height - ((amplitude - min) / scale)))
            Else
                yAxis = CInt(Math.Truncate(0 + ((max - amplitude) / scale)))
            End If
            If yAxis < 0 Then
                yAxis = 0
            End If
            If yAxis > height Then
                yAxis = height
            End If
            pen.Color = Pencolour
            offScreenDC.DrawLine(pen, xAxis, height, xAxis, yAxis)
        Next
        offScreenDC.DrawString("Min: " & minHz.ToString(".#") & " Hz (±" & scaleHz.ToString(".#") & ") = " & min.ToString(".###"), font, brush, 0 + 1, 0 + 1)
        offScreenDC.DrawString("Max: " & maxHz.ToString(".#") & " Hz (±" & scaleHz.ToString(".#") & ") = " & max.ToString(".###"), font, brush, 0 + 1, 0 + 18)

        ' Clean up
        pictureBox.Image = canvas
        offScreenDC.Dispose()
    End Sub

    Public Sub RenderTimeDomain(ByRef pictureBox As PictureBox, ByRef RawSampleData As Double())
        ' Set up for drawing
        Dim canvas As New Bitmap(pictureBox.Width, pictureBox.Height)
        Dim offScreenDC As Graphics = Graphics.FromImage(canvas)
        Dim pen As Pen = New System.Drawing.Pen(Color.WhiteSmoke)

        ' Determine channnel boundries
        Dim width As Integer = canvas.Width
        Dim height As Integer = canvas.Height
        Dim center As Double = height \ 2

        ' Draw left channel
        Dim scale As Double = 0.5 * height / 32768
        ' a 16 bit sample has values from -32768 to 32767
        Dim xPrev As Integer = 0, yPrev As Integer = 0
        For x As Integer = 0 To width - 1
            Dim y As Integer = CInt(Math.Truncate(center + (RawSampleData(CInt(RawSampleData.Length / width * x)) * scale)))
            If x = 0 Then
                xPrev = 0
                yPrev = y
            Else
                pen.Color = Color.Green
                offScreenDC.DrawLine(pen, xPrev, yPrev, x, y)
                xPrev = x
                yPrev = y
            End If
        Next

        ' Clean up
        pictureBox.Image = canvas
        offScreenDC.Dispose()
    End Sub






End Class
