﻿Imports System
Imports System.Xml
Imports System.IO

Public Class SampleLoader

    Implements IDisposable

    Private disposedValue As Boolean


    Public Structure DataPoint
        Public Seq As Integer
        Public Val As Double
    End Structure

    Public Structure Points
        Public Value As List(Of DataPoint)
    End Structure


    Public Structure CapturedData
        Public Traces As List(Of Points)
        Public triggerMode As String
        Public triggerKind As String
        Public triggerLevel As String
        Public triggerSensitivity As String
        Public attenuation As String
        Public voltageDiv As String
        Public timeDiv As String
        Public sampleDiv As String
        Public firmware As String
        Public fileNumber As String
        Public triggerIndex As Integer
        Public sampleCount As Integer
        Public timeRange As Double
        Public MinValue As Double
        Public MaxValue As Double
        Public MinIndex As Integer
        Public MaxIndex As Integer
        Public SampleFrequency As Double
        Public ValidData As Boolean
    End Structure

    Public Function LoadXMLFile(ByVal strFilename As String) As CapturedData

        Dim oFile As New XmlDocument

        LoadXMLFile = New CapturedData


        Try

            oFile.Load(strFilename)

            Dim oDataPoints As XmlNodeList = oFile.GetElementsByTagName("Point")
            Dim oDataProfile As XmlNodeList = oFile.GetElementsByTagName("Profile")
            Dim oPoint As New DataPoint
            LoadXMLFile.Traces = New List(Of Points)

            LoadXMLFile.triggerMode = CStr(oDataProfile(0).Item("triggerMode").InnerText)
            LoadXMLFile.triggerKind = CStr(oDataProfile(0).Item("triggerKind").InnerText)
            LoadXMLFile.triggerLevel = CStr(oDataProfile(0).Item("triggerLevel").InnerText)
            LoadXMLFile.triggerSensitivity = CStr(oDataProfile(0).Item("triggerSensitivity").InnerText)
            LoadXMLFile.attenuation = CStr(oDataProfile(0).Item("attenuation").InnerText)
            LoadXMLFile.voltageDiv = CStr(oDataProfile(0).Item("voltageDiv").InnerText)
            LoadXMLFile.timeDiv = CStr(oDataProfile(0).Item("timeDiv").InnerText)
            LoadXMLFile.sampleDiv = CStr(oDataProfile(0).Item("sampleDiv").InnerText)
            LoadXMLFile.firmware = CStr(oDataProfile(0).Item("firmware").InnerText)
            LoadXMLFile.fileNumber = CStr(oDataProfile(0).Item("fileNumber").InnerText)
            LoadXMLFile.triggerIndex = CInt(oDataProfile(0).Item("triggerIndex").InnerText)
            LoadXMLFile.sampleCount = CInt(oDataProfile(0).Item("sampleCount").InnerText)
            LoadXMLFile.timeRange = CDbl(oDataProfile(0).Item("timeRange").InnerText)
            LoadXMLFile.SampleFrequency = LoadXMLFile.sampleCount / LoadXMLFile.timeRange

            LoadXMLFile.MinValue = Double.MaxValue
            LoadXMLFile.MaxValue = Double.MinValue

            Dim oPoints As New Points
            oPoints.Value = New List(Of DataPoint)
            Dim intPointer As Integer = 0
            For Each oDataPoint As XmlNode In oDataPoints
                oPoint = New DataPoint
                oPoint.Seq = CInt(oDataPoint.FirstChild.InnerText)
                oPoint.Val = CDbl(oDataPoint.LastChild.InnerText)

                If oPoint.Val < LoadXMLFile.MinValue Then
                    LoadXMLFile.MinValue = oPoint.Val
                    LoadXMLFile.MinIndex = intPointer
                End If

                If oPoint.Val > LoadXMLFile.MaxValue Then
                    LoadXMLFile.MaxValue = oPoint.Val
                    LoadXMLFile.MaxIndex = intPointer
                End If

                oPoints.Value.Add(oPoint)

                intPointer += 1

            Next

            LoadXMLFile.Traces.Add(oPoints)

            LoadXMLFile.ValidData = True


        Catch ex As Exception
            LoadXMLFile.ValidData = False
        End Try


    End Function


#Region "IDisposable Support"


    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
            End If
        End If
        Me.disposedValue = True
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

#End Region

End Class
